(*****************************************************************)
(*                                                               *)
(*  ASDL to Caml translator -- support library                   *)
(*  Copyright (c) 2000 by Dmitry Lomov (dsl@tepkom.ru)           *)
(*  Distributed under GNU Library Public License                 *)
(*                                                               *)
(*****************************************************************)

(* $Id: asdlBase.ml,v 1.1.1.1 2000/12/05 09:06:52 dsl Exp $ *)

type identifier = Identifier.t

let write_int x s =
  let set_neg_bit = x < 0 in
  let x = if set_neg_bit then -x else x in
  let rec loop x =
    if x > 63
    then 
      let v = ((x land 0x7F) lor 0x80) in
      (output_byte s v; loop (x lsr 7))
    else x in
  let x = loop x in
  let x = if set_neg_bit then x lor 0x40 else x in
  output_byte s x

let write_tag x s =
  write_int x s

let write_string x s =
  let l = (String.length x) in
  write_int l s;
  for i = 0 to l-1 do
    output_char s x.[i]
  done
  
let write_identifier x s =
  write_string (Identifier.to_const_string x) s

let read_int s = 
  let rec loop acc shift =
    let x = input_byte s in
    if x land 0x80 == 0
    then 
      let acc = acc lor ((x land 0x3F) lsl shift) in
      if x land 0x40 != 0
      then -acc
      else acc
    else
      loop (acc lor ((x land 0x7F) lsl shift)) (shift + 7) in
  loop 0 0

let read_tag s = read_int s

let read_string s =
  let l = read_int s in
  let res = String.create l in
  for i = 0 to l-1 do
    res.[i] <- input_char s
  done;
  res

let read_identifier s = 
  Identifier.from_string (read_string s)

let write_list f l s =
  write_tag (List.length l) s;
  List.iter (fun x -> f x s) l

let write_option f v s =
  match v with
    None -> write_tag 0 s
  | Some v -> write_tag 1 s; f v s

let read_list f s =
  let rec read_next n res =
    if n = 0
    then res 
    else read_next (n-1) ((f s)::res) in
  let l = read_tag s in
  List.rev (read_next l [])

exception PicklerError

let die () = 
  raise PicklerError

let read_option f s =
  match read_tag s with
    0 -> None
  | 1 -> Some (f s)
  | _ -> die ()

    
