(*****************************************************************)
(*                                                               *)
(*  ASDL to Caml translator                                      *)
(*  Copyright (c) 2000 by Dmitry Lomov (dsl@tepkom.ru)           *)
(*  Distributed under GNU Public License                         *)
(*                                                               *)
(*****************************************************************)

(* $Id: asdllex.mll,v 1.1.1.1 2000/12/05 09:06:52 dsl Exp $ *)

{
open Asdlpars

let line_no = ref 0
let line_start = ref 0

let new_line lexbuf =
  line_no := !line_no + 1; 
  line_start := Lexing.lexeme_end lexbuf
} 

let idchar = ['A'-'Z']|['a'-'z']|['0'-'9']|'_' 

rule token = parse
  [ ' ' '\t' ]   { token lexbuf } (* Skip whitespace *)
| "--" [^'\n']* '\n' { new_line lexbuf; token lexbuf }
| ':' [^'\n']* '\n' 
                 {
		   new_line lexbuf; 
		   LITTEXT 
		 }
| "%%\n" _* "%%\n"   
                 { 
		   let s = Lexing.lexeme lexbuf in
		   for i = 0 to (String.length s) - 1 do
		     if s.[i] == '\n' then line_no := !line_no+1
		   done;
		   line_start := Lexing.lexeme_end lexbuf;
		   LITTEXT 
		 }
| [ '\n' ]       { new_line lexbuf; token lexbuf } 
| "module"       { MODULE }
| "imports"      { IMPORTS }
| "attributes"   { ATTRIBUTES }
| "view"         { VIEW }
| "<="           { LARROW2 }
| '('            { LPAR }
| ')'            { RPAR }
| '{'            { LCURL }
| '}'            { RCURL }
| ','            { COMMA }
| '.'            { DOT }
| '|'            { VERT }
| '?'            { QUEST }
| '*'            { ASTERISK }
| '='            { EQ }
| eof            { EOF }
| [ 'a'-'z'] idchar * { LIDENT(Lexing.lexeme lexbuf) }
| [ 'A'-'Z'] idchar * { UIDENT(Lexing.lexeme lexbuf) } 











