/*****************************************************************
 *                                                               *
 *  ASDL to Caml translator                                      *
 *  Copyright (c) 2000 by Dmitry Lomov (dsl@tepkom.ru)           *
 *  Distributed under GNU Public License                         *
 *                                                               *
 *****************************************************************/
 
/* $Id: asdlpars.mly,v 1.1.1.1 2000/12/05 09:06:52 dsl Exp $ */
 
/* ASDL syntax */
%{
open AsdlDef
%} 

/*  Keywords  */
%token MODULE
%token IMPORTS
%token ATTRIBUTES
%token LPAR
%token RPAR
%token LCURL
%token RCURL
%token COMMA
%token VERT
%token QUEST
%token ASTERISK
%token DOT
%token EQ
%token VIEW
%token LARROW2
%token LITTEXT
%token EOF

/* Identifiers */
%token <string> LIDENT
%token <string> UIDENT

%type <unit> moduleDef
%type <id list> imports
%type <typeDef list> definitions
%type <typeDef> typedef
%type <typeExpr> typexpr
%type <constructor> constructor
%type <field list> fields
%type <field> field
%type <typeId> typeid
%type <id> id
%type <unit> def
%type <unit> def_list

%start def_list

%% 

id : 
  LIDENT { $1 }
| UIDENT { $1 };

def_list : 
  EOF { () }
| def def_list { () };

def : 
  moduleDef { () }
| viewDef { () };

moduleDef : 
  MODULE id LCURL definitions RCURL 
  { Gen.genModule ($2,[],$4) }
| MODULE id LPAR imports RPAR LCURL definitions RCURL
  { Gen.genModule ($2, $4, $7) };

importsGroup : IMPORTS id { $2 }; 

imports : 
  importsGroup { [$1] }
| importsGroup imports { $1::$2 };

definitions : 
  typedef { [$1] }
| typedef definitions { $1::$2 };

typedef :
  LIDENT EQ typexpr { ($1, $3) };

typexpr :
  fields { (Product $1) }
| sum_t { let a,b = $1 in Sum(a,b) };

sum_t :
  constructor_list { ($1, []) }
| constructor_list ATTRIBUTES fields { ($1, $3) };

constructor_list :
  constructor { [$1] }
| constructor VERT constructor_list { $1::$3 };

constructor :
  UIDENT { ($1, []) }
| UIDENT fields { ($1,$2) };

fields :
  LPAR field_list RPAR { $2 };

field_list :
  field { [$1] }
| field COMMA field_list { $1::$3 };

typeid :
  id DOT LIDENT { ((Some $1),$3) }
| LIDENT { (None, $1) }; 

field : 
  typeid fieldid { let m,id = $2 in m, id, Straight, $1 } 
| typeid QUEST fieldid { let m,id = $3 in m, id, Optional, $1 }
| typeid ASTERISK fieldid { let m,id = $3 in m, id, Sequence, $1 };

fieldid : { Generated, "" }
| id { User, $1 }

viewDef : VIEW id LCURL viewEntryList RCURL { () };

viewEntryList : 
  viewEntry { () }
| viewEntry viewEntryList { () };

viewEntity :
  id DOT LIDENT { () }
| id DOT UIDENT { () }
| MODULE id { () };

viewProp : LARROW2 id { () }

viewEntry : 
  viewEntity viewProp  LITTEXT { () }
| LCURL viewEntityList RCURL viewProp LITTEXT { () }
| viewProp LCURL viewPropList RCURL { () }
| viewEntity LARROW2 LCURL idList RCURL { () }
| LCURL viewEntityList RCURL LARROW2 LCURL idList RCURL { () }

viewEntityList :
  viewEntity { () } 
| viewEntity viewEntityList { () }

viewPropList :
  { () }
| viewEntity LITTEXT viewPropList { () }

idList : 
  { () }
| id LITTEXT idList { () }
    






