(*****************************************************************)
(*                                                               *)
(*  ASDL to Caml translator                                      *)
(*  Copyright (c) 2000 by Dmitry Lomov (dsl@tepkom.ru)           *)
(*  Distributed under GNU Public License                         *)
(*                                                               *)
(*****************************************************************)

(* $Id: main.ml,v 1.1.1.1 2000/12/05 09:06:52 dsl Exp $ *)

let arguments =
  [
   ("-s", 
    Arg.Unit(fun () -> Params.v.Params.separate <- true), 
    "generate .ml and .mli files for each ASDL module");
   ("-f",
    Arg.Unit(fun () -> Params.v.Params.filenames <- true),
    "output names of created files to stdout")]

let processASDL fn =
  Params.v.Params.cur_file_name <- fn;
  try
    let fin = open_in fn in    
    let lexbuf = Lexing.from_channel fin in
    try    
      Asdlpars.def_list Asdllex.token lexbuf
    with
      Parsing.Parse_error ->
	Params.msg Params.Error 
	  ("Syntax error near token "^(Lexing.lexeme lexbuf))
	  (Some(
	   !Asdllex.line_no, 
	   ((Lexing.lexeme_start lexbuf) - !Asdllex.line_start),
	   ((Lexing.lexeme_end lexbuf) - !Asdllex.line_start)))
    | _ -> 
	Params.msg Params.Error 
	  ("Internal error near token "^(Lexing.lexeme lexbuf))
	  (Some(
	   !Asdllex.line_no, 
	   ((Lexing.lexeme_start lexbuf) - !Asdllex.line_start),
	   ((Lexing.lexeme_end lexbuf) - !Asdllex.line_start)))
  with
    Sys_error s ->
      Params.msg Params.Error s None
	
let program_id_line = 
  "ASDL to O'Caml translator\nUsage: asdl2caml [options] file...\nOptions are:"

let help () =
  Printf.fprintf stderr "asdl2caml: missing arguments.";
  Printf.fprintf stderr " Type \"asdl2caml -help\" for help.\n"

let _ =
  if Array.length Sys.argv < 2
  then 
    help ()
  else
    Arg.parse arguments processASDL program_id_line
      
  






