(*****************************************************************)
(*                                                               *)
(*  ASDL to Caml translator                                      *)
(*  Copyright (c) 2000 by Dmitry Lomov (dsl@tepkom.ru)           *)
(*  Distributed under GNU Public License                         *)
(*                                                               *)
(*****************************************************************)

(* $Id: naming.ml,v 1.1.1.1 2000/12/05 09:06:52 dsl Exp $ *)

open AsdlDef 

let keywords = Hashtbl.create 51
let _ = List.iter (fun (kwd, tok) -> Hashtbl.add keywords kwd tok) [
    "and", "and'";
    "as", "as'";
    "assert", "assert'";
    "begin", "begin'";
    "class", "class'";
    "constraint", "constraint'";
    "do", "do'";
    "done", "done'";
    "downto", "downto'";
    "else", "else'";
    "end", "end'";
    "exception", "exception'";
    "external", "external'";
    "false", "false'";
    "for", "for'";
    "fun", "fun'";
    "function", "function'";
    "functor", "functor'";
    "if", "if'";
    "in", "in'";
    "include", "include'";
    "inherit", "inherit'";
    "initializer", "initializer'";
    "lazy", "lazy'";
    "let", "let'";
    "match", "match'";
    "when", "when'";
    "method", "method'";
    "module", "module'";
    "mutable", "mutable'";
    "new", "new'";
    "not", "not'";
    "object", "object'";
    "of", "of'";
    "open", "open'";
    "or", "or'";
    "parser", "parser'";
    "private", "private'";
    "rec", "rec'";
    "sig", "sig'";
    "struct", "struct'";
    "then", "then'";
    "to", "to'";
    "true", "true'";
    "try", "try'";
    "type", "type'";
    "val", "val'";
    "virtual", "virtual'";
    "when", "when'";
    "while", "while'";
    "with", "with'"]

let rename_type tid = 
  try
    Hashtbl.find keywords tid
  with 
    Not_found -> tid

let typeId_naming (m, typ) =
  (match m with 
    Some s -> Some (String.capitalize s)
  | None -> None),
  rename_type typ
  
let rec field_naming i prefix = function 
    [] -> [] 
  | (idMod, id, typeMod, typeId)::t -> 
      let id1, i1 = 
	match idMod with
	  User -> (prefix^id), i
	| Generated -> prefix^(string_of_int i), i+1 in
      (idMod, id1, typeMod, (typeId_naming typeId))
      ::(field_naming i1 prefix t)

let type_expr_naming tid typ =
  let prefix = tid^"_" in
  match typ with
    Product l ->
      (Product (field_naming 1 prefix l))
  | Sum (c, l) ->
      let c1 =
	List.map (fun (cid, l) -> (cid, (field_naming 1 ("_"^cid^"_") l))) c in
      Sum(c1, (field_naming 1 prefix l))

let (type_naming : typeDef -> typeDef) =
  fun (id, expr) ->
    let tid = rename_type id in
    tid, type_expr_naming tid expr

let (naming : moduleDef -> moduleDef) = 
  fun (id, imports, tlist) ->
    (String.capitalize id), 
    (List.map String.capitalize imports),
    (List.map type_naming tlist)
